Enum SmtProofError
pub enum SmtProofError {
InvalidMerklePathLength(usize),
InvalidKeyForProof,
ValueMismatch {
expected: Word,
actual: Word,
},
ConflictingRoots {
expected_root: Word,
actual_root: Word,
},
ValuePresent {
key: Word,
value: Word,
},
}Expand description
Errors that can occur when validating SMT proofs.
Variants
InvalidMerklePathLength(usize)
The length of the provided Merkle path is not SMT_DEPTH.
InvalidKeyForProof
The key maps to a different leaf index than the proof’s leaf.
ValueMismatch
The value does not match the value in the leaf for the given key.
ConflictingRoots
The computed root does not match the expected root.
ValuePresent
The key-value pair exists in the tree.