Enum ExecutionError
pub enum ExecutionError {
Show 13 variants
AceChipError {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
error: AceError,
},
AdviceError {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
err: AdviceError,
},
CycleLimitExceeded(u32),
EventError {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
event_id: EventId,
event_name: Option<EventName>,
error: Box<dyn Error + Sync + Send>,
},
Internal(&'static str),
TraceLenExceeded(usize),
MemoryError {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
err: MemoryError,
},
MemoryErrorNoCtx(MemoryError),
OperationError {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
err: OperationError,
},
OutputStackOverflow(usize),
ProcedureNotFound {
label: SourceSpan,
source_file: Option<Arc<SourceFile>>,
root_digest: Word,
},
ProvingError(String),
HostError(HostError),
}Variants
AceChipError
AdviceError
CycleLimitExceeded(u32)
EventError
Internal(&'static str)
TraceLenExceeded(usize)
This means trace generation would go over the configured row limit.
In parallel trace building, this is used for core-row prechecks and chiplet overflow.
MemoryError
Memory error with source context for diagnostics.
Use MemoryResultExt::map_mem_err to convert Result<T, MemoryError> with context.
MemoryErrorNoCtx(MemoryError)
Memory error without source context (for internal operations like FMP initialization).
Use ExecutionError::MemoryErrorNoCtx for memory errors that don’t have error context
available (e.g., during call/syscall context initialization).
OperationError
OutputStackOverflow(usize)
ProcedureNotFound
ProvingError(String)
HostError(HostError)