Struct ProgramInfo
pub struct ProgramInfo { /* private fields */ }Expand description
A program information set consisting of its MAST root and set of kernel procedure roots used for its compilation.
This will be used as public inputs of the proof so we bind its verification to the kernel and root used to execute the program. This way, we extend the correctness of the proof to the security guarantees provided by the kernel. We also allow the user to easily prove the membership of a given kernel procedure for a given proof, without compromising its zero-knowledge properties.
Implementations
impl ProgramInfo
impl ProgramInfo
pub const fn new(program_hash: Word, kernel: Kernel) -> ProgramInfo
pub const fn new(program_hash: Word, kernel: Kernel) -> ProgramInfo
Creates a new instance of a program info.
pub const fn program_hash(&self) -> &Word
pub const fn program_hash(&self) -> &Word
Returns the program hash computed from its code block root.
pub const fn kernel(&self) -> &Kernel
pub const fn kernel(&self) -> &Kernel
Returns the program kernel used during the compilation.
pub fn kernel_procedures(&self) -> &[Word]
pub fn kernel_procedures(&self) -> &[Word]
Returns the list of procedures of the kernel used during the compilation.