pub struct PublicInputs { /* private fields */ }Implementations
Sourceimpl PublicInputs
impl PublicInputs
Sourcepub fn new(
program_info: ProgramInfo,
stack_inputs: StackInputs,
stack_outputs: StackOutputs,
pc_transcript_state: Word,
) -> PublicInputs
pub fn new( program_info: ProgramInfo, stack_inputs: StackInputs, stack_outputs: StackOutputs, pc_transcript_state: Word, ) -> PublicInputs
Creates a new instance of PublicInputs from program information, stack inputs and outputs,
and the precompile transcript state (capacity of an internal sponge).
pub fn stack_inputs(&self) -> StackInputs
pub fn stack_outputs(&self) -> StackOutputs
pub fn program_info(&self) -> ProgramInfo
Sourcepub fn pc_transcript_state(&self) -> Word
pub fn pc_transcript_state(&self) -> Word
Returns the precompile transcript state.
Sourcepub fn to_air_inputs(&self) -> (Vec<Felt>, Vec<Felt>)
pub fn to_air_inputs(&self) -> (Vec<Felt>, Vec<Felt>)
Returns the fixed-length public values and the variable-length kernel procedure digests
as a flat slice of Felts.
The fixed-length public values layout is: [0..4] program hash [4..20] stack inputs [20..36] stack outputs [36..40] precompile transcript state
The kernel procedure digests are returned as a single flat Vec<Felt> (concatenated
words), to be passed as a single variable-length public input slice to the verifier.
Sourcepub fn to_elements(&self) -> Vec<Felt>
pub fn to_elements(&self) -> Vec<Felt>
Converts public inputs into a vector of field elements (Felt) in the canonical order:
- program info elements (including kernel procedure hashes)
- stack inputs
- stack outputs
- precompile transcript state