Skip to main contentCrate miden_prover
Source - config
- STARK configuration factories for different hash functions.
- crypto
- field
- serde
- utils
- AdviceInputs
- Inputs container to initialize advice provider for the execution of Miden VM programs.
- ExecutionProof
- A proof of correct execution of Miden VM.
- ProcessorAir
- Miden VM Processor AIR implementation.
- ProvingOptions
- A set of parameters specifying how Miden VM execution proofs are to be generated.
- StackInputs
- Defines the initial state of the VM’s operand stack.
- StackOutputs
- Defines the final state of the VM’s operand stack at the end of program execution.
- Word
- A unit of data consisting of 4 field elements.
- DeserializationError
- Defines errors which can occur during deserialization.
- ExecutionError
- HashFunction
- A hash function used during STARK proof generation.
- InputError
- Host
- Defines an interface by which the VM can interact with the host.
- aux_trace_to_row_major
- Converts an auxiliary trace from column-major to row-major format.
- execution_trace_to_row_major
- Converts the main trace from column-major (ExecutionTrace) to row-major (Plonky3) format.
- prove
- Executes and proves the specified
program and returns the result together with a STARK-based
proof of the program’s execution. - prove_sync
- Synchronous wrapper for the async
prove() function.