Struct Signature
pub struct Signature { /* private fields */ }Expand description
ECDSA signature over secp256k1 curve using Keccak to hash the messages when signing.
Serialization Formats
This implementation supports 2 serialization formats:
Custom Format (66 bytes):
- Bytes 0-31: r component (32 bytes, big-endian)
- Bytes 32-63: s component (32 bytes, big-endian)
- Byte 64: recovery ID (v) - values 0-3
SEC1 Format (64 bytes):
- Bytes 0-31: r component (32 bytes, big-endian)
- Bytes 32-63: s component (32 bytes, big-endian)
- Note: Recovery ID
Implementations
impl Signature
impl Signature
pub fn v(&self) -> u8
pub fn v(&self) -> u8
Returns the v component of this signature, which is a u8 representing the recovery id.
pub fn verify(&self, message: Word, pub_key: &PublicKey) -> bool
pub fn verify(&self, message: Word, pub_key: &PublicKey) -> bool
Verifies this signature against a message and public key.
pub fn to_sec1_bytes(&self) -> [u8; 64]
pub fn to_sec1_bytes(&self) -> [u8; 64]
Converts signature to SEC1 format (standard 64-byte r||s format).
This format is the standard one used by most ECDSA libraries but loses the recovery ID.
pub fn from_sec1_bytes_and_recovery_id(
bytes: [u8; 64],
recovery_id: u8,
) -> Result<Signature, DeserializationError>
pub fn from_sec1_bytes_and_recovery_id( bytes: [u8; 64], recovery_id: u8, ) -> Result<Signature, DeserializationError>
Creates a signature from SEC1 format bytes with a given recovery id.
Arguments
bytes- 64-byte array containing r and s componentsrecovery_id- recovery ID (0-3)