FutureMaybeSend

Trait FutureMaybeSend 

pub trait FutureMaybeSend<O>: Future<Output = O> + Send { }
Alias for a Future

Unless the compilation target family is wasm, we add Send to the required bounds. For wasm compilation targets there is no Send bound.

Implementors



impl<T, O> FutureMaybeSend<O> for T
where T: Future<Output = O> + Send,

Available on non-target_family=wasm only.